# ########################################################################### #
# #    Copyright (c) 2019-2020, California Institute of Technology.
# #    All rights reserved.  Based on Government Sponsored Research under
# #    contracts NNN12AA01C, NAS7-1407 and/or NAS7-03001.
# #
# #    Redistribution and use in source and binary forms, with or without
# #    modification, are permitted provided that the following conditions
# #    are met:
# #      1. Redistributions of source code must retain the above copyright
# #         notice, this list of conditions and the following disclaimer.
# #      2. Redistributions in binary form must reproduce the above copyright
# #         notice, this list of conditions and the following disclaimer in
# #         the documentation and/or other materials provided with the
# #         distribution.
# #      3. Neither the name of the California Institute of
# #         Technology (Caltech), its operating division the Jet Propulsion
# #         Laboratory (JPL), the National Aeronautics and Space
# #         Administration (NASA), nor the names of its contributors may be
# #         used to endorse or promote products derived from this software
# #         without specific prior written permission.
# #
# #    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# #    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# #    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# #    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CALIFORNIA
# #    INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# #    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# #    TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# #    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# #    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# #    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# #    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# #
# # ########################################################################### #
# #    EXOplanet Transit Interpretation Code (EXOTIC)
# #    # NOTE: See companion file version.py for version info.
# # ########################################################################### #

from tkinter import filedialog
import tkinter as tk
import exotic
import os
import json
import ast
import subprocess
from datetime import datetime

try:  # filters
    from .api.filters import fwhm as photometric_filters
except ImportError:  # package import
    from api.filters import fwhm as photometric_filters

try:  # simple version
    from .version import __version__
except ImportError:  # package import
    from version import __version__

class FolderSelect(tk.Frame):
    def __init__(self,parent=None,folderDescription="",default_text="",**kw):
        tk.Frame.__init__(self,master=parent,**kw)
        self.folderPath = tk.StringVar()
        self.lblName = tk.Label(self, text=folderDescription, anchor=tk.W)
        self.lblName.grid(row=0,column=0)
        self.entPath = tk.Entry(self, textvariable=self.folderPath)
        self.entPath.insert(tk.END, default_text)
        self.entPath.grid(row=0,column=1)
        self.btnFind = tk.Button(self, text="Browse Folder",command=self.setFolderPath, anchor=tk.W)
        self.btnFind.grid(row=0,column=2)
    def setFolderPath(self):
        folder_selected = filedialog.askdirectory()
        self.folderPath.set(folder_selected)
    @property
    def folder_path(self):
        return self.folderPath.get()

class FileSelect(tk.Frame):
    def __init__(self,parent=None,folderDescription="",default_text="",**kw):
        tk.Frame.__init__(self,master=parent,**kw)
        self.filePath = tk.StringVar()
        self.lblName = tk.Label(self, text=folderDescription, anchor=tk.W)
        self.lblName.grid(row=0,column=0)
        self.entPath = tk.Entry(self, textvariable=self.filePath)
        self.entPath.insert(tk.END, default_text)
        self.entPath.grid(row=0,column=1)
        self.btnFind = tk.Button(self, text="Browse",command=self.setFilePath, anchor=tk.W)
        self.btnFind.grid(row=0,column=2)
    def setFilePath(self):
        file_selected = filedialog.askopenfilename()
        self.filePath.set(file_selected)
    @property
    def file_path(self):
        return self.filePath.get()

def main():

    root = tk.Tk()
    root.title(f"EXOTIC v{__version__}")

    tk.Label(root,
            text="Welcome to EXOTIC!",
            justify = tk.LEFT,
            font=("Arial", 36),
            padx = 20).pack(anchor=tk.CENTER)
    tk.Label(root,
            text="the EXOplanet Transit Interpretation Code",
            justify = tk.LEFT,
            font=("Arial", 22),
            padx = 20).pack(anchor=tk.CENTER)
    tk.Label(root,
            text=f"Version {__version__}",
            justify = tk.LEFT,
            padx = 20).pack(anchor=tk.CENTER)

    tk.Label(root,
            text="Copyright (c) 2021, California Institute of Technology. All rights reserved.\nBased on Government Sponsored Research under contracts NNN12AA01C, NAS7-1407 and/or NAS7-03001.",
            font=("Arial", 12),
            justify = tk.CENTER,
            padx = 20).pack(anchor=tk.CENTER)

    # Button for closing
    exit_button = tk.Button(root, text="Next", command=root.destroy)
    exit_button.pack(anchor=tk.CENTER)

    root.mainloop()

    root = tk.Tk()
    root.title(f"EXOTIC v{__version__}")

    reduction_opt = tk.IntVar()
    tk.Label(root,
            text="\nHow would you like to run EXOTIC?",
            justify = tk.LEFT,
            font=("Helvetica 14 bold"),
            padx = 20).pack(anchor=tk.W)

    tk.Radiobutton(root,
                   text="Real Time Reduction - COMING SOON\n(for analyzing your data while simultaneously observing)",
                   justify = tk.LEFT,
                   padx = 20,
                   variable=reduction_opt,
                   state=tk.DISABLED,
                   value=1).pack(anchor=tk.W)

    tk.Radiobutton(root,
                   text="Complete Reduction\n(for analyzing your data after an observing run)",
                   justify = tk.LEFT,
                   padx = 20,
                   variable=reduction_opt,
                   value=2).pack(anchor=tk.W)

    # Button for closing
    exit_button = tk.Button(root, text="Next", command=root.destroy)
    exit_button.pack(anchor=tk.E)

    root.mainloop()


    if reduction_opt.get() == 2:
        root = tk.Tk()
        root.title(f"EXOTIC v{__version__}")

        fitsortext = tk.IntVar()

        tk.Label(root,
                 text="""How do you want to run EXOTIC?""",
                 font=("Helvetica 14 bold"),
                 justify=tk.LEFT,
                 padx=20).pack()

        tk.Radiobutton(root,
                       text="Analyze your image files",
                       padx=20,
                       variable=fitsortext,
                       value=1).pack(anchor=tk.W)

        tk.Radiobutton(root,
                       text="Start with pre-reduced data in a .txt format - COMING SOON",
                       padx=20,
                       variable=fitsortext,
                       state=tk.DISABLED,
                       value=2).pack(anchor=tk.W)

        # Button for closing
        exit_button = tk.Button(root, text="Next", command=root.destroy)
        exit_button.pack(pady=20,anchor=tk.E)

        root.mainloop()



        if fitsortext.get() == 1:

            # First ask user how they want to enter the observing information
            root = tk.Tk()
            obsinfo = tk.StringVar()

            root.title(f"EXOTIC v{__version__}")

            tk.Label(root,
                    text="""How would you like to input your observing information?""",
                    justify=tk.LEFT,
                    font=("Helvetica 14 bold"),
                    padx=20).pack()

            tk.Radiobutton(root,
                        text="Manually",
                        padx=20,
                        variable=obsinfo,
                        value="manual").pack(anchor=tk.W)

            tk.Radiobutton(root,
                        text="From a pre-existing input file (e.g., inits.json)",
                        padx=20,
                        variable=obsinfo,
                        value='inits').pack(anchor=tk.W)

            # Button for closing
            exit_button = tk.Button(root, text="Next", command=root.destroy)
            exit_button.pack(pady=20,anchor=tk.E)

            root.mainloop()

            input_data = {} # for saving entries

            if obsinfo.get() == "manual":
                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                initparams = tk.IntVar()

                window_label = tk.Label(root,
                                        text="""Please enter the following information about your observation:""",
                                        font=("Helvetica 14 bold"),
                                        justify=tk.CENTER,
                                        padx=20)  # .pack()
                window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                # Set up rows + columns
                i = 1;
                j = 0

                folderPath = tk.StringVar()

                # # "Directory with FITS files": "sample-data/HatP32Dec202017",
                FITS_dir = FolderSelect(root, "Folder with your FITS files")
                FITS_dir.grid(row=i)
                i += 1

                # "Directory to Save Plots": "sample-data/",
                save_dir = FolderSelect(root, "Folder to save EXOTIC output")
                save_dir.grid(row=i)
                i += 1

                window_label = tk.Label(root,
                                        text="""Please your calibration file information\n(Note: each calibration type must be in its OWN, SEPARATE folder)""",
                                        # font=("Helvetica 14 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=i, column=0, sticky=tk.N, pady=6)
                i += 1

                # "Directory of Flats": null,
                flats_dir = FolderSelect(root, "Folder with your flats (null if none)", "null")
                flats_dir.grid(row=i)
                i += 1

                # "Directory of Darks": null,
                darks_dir = FolderSelect(root, "Folder with your darks (null if none)", "null")
                darks_dir.grid(row=i)
                i += 1

                # "Directory of Biases": null,
                biases_dir = FolderSelect(root, "Folder with your biases (null if none)", "null")
                biases_dir.grid(row=i)
                i += 1

                # "Directory of Biases": null,
                #biases_dir = FolderSelect(root, "Folder with your biases (null if none)", "null")
                #biases_dir.grid(row=i)
                #i += 1

                #
                # obscode2_label = TextInput(root, "AAVSO Obs Code", "RTZ")
                # obscode2_label.grid(row=1)
                # i += 1


                #             # "AAVSO Observer Code (N/A if none)": "RTZ",
                obscode_label = tk.Label(root, text="AAVSO Observer Code (N/A if none)", justify=tk.LEFT)
                obscode_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                obscode_entry.insert(tk.END, "N/A")
                obscode_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                obscode_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                #
                #             # "Secondary Observer Codes (N/A if none)": "N/A",
                secondobscode_label = tk.Label(root, text="Secondary Observer Codes (N/A if none)", justify=tk.LEFT)
                secondobscode_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                secondobscode_entry.insert(tk.END, "N/A")
                secondobscode_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                secondobscode_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                #             # "Observation date": "17-December-2017",
                obsdate_label = tk.Label(root, text="Observation date (e.g. DAY-MONTH-YEAR)", justify=tk.LEFT)
                obsdate_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                obsdate_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                obsdate_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                # #             "Obs. Latitude": "+32.41638889",
                lat_label = tk.Label(root, text="Obs. Latitude (e.g. +32.41)", justify=tk.LEFT)
                lat_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                lat_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                lat_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                # #             "Obs. Longitude": "-110.73444444",
                long_label = tk.Label(root, text="Obs. Longitude (e.g. -110.74) ", justify=tk.LEFT)
                long_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                long_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                long_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                # #             "Obs. Elevation (meters)": 2616,
                elevation_label = tk.Label(root, text="Obs. Elevation [meters]", justify=tk.LEFT)
                elevation_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                elevation_entry.insert(tk.END, "0")
                elevation_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                elevation_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                # #             "Camera Type (CCD or DSLR)": "CCD",
                cameratype_label = tk.Label(root, text="Camera Type (e.g. CCD or DSLR)", justify=tk.LEFT)
                cameratype_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                cameratype_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                cameratype_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1
                #
                # #             "Pixel Binning": "1x1",
                pixbin_label = tk.Label(root, text="Pixel Binning  (e.g 1x1)", justify=tk.LEFT)
                pixbin_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                pixbin_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                pixbin_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #     "Pixel Scale (Ex: 5.21 arcsecs/pixel)": null,
                pixscale_label = tk.Label(root, text="Pixel Scale (null if unknown)", justify=tk.LEFT)
                pixscale_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                pixscale_entry.insert(tk.END, "5.21 arcsecs/pixel")
                pixscale_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                pixscale_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                # # #             "Filter Name (aavso.org/filters)": "V",
                # filtername_label = tk.Label(root, text="Filter Name (please see aavso.org/filters)", justify=tk.LEFT)
                # filtername_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                # filtername_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                # filtername_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                # i += 1

                # choices = ['one', 'two', 'three']
                choices = [item for sublist in photometric_filters for item in sublist]
                choices = sorted(set(choices)) # sort and list unique values
                filteroptions = tk.StringVar(root)
                filteroptions.set(choices[0])  # default value

                l3 = tk.Label(root, text='Filter (use N/A for custom)', width=25)
                l3.grid(row=i, column=0)

                om1 = tk.OptionMenu(root, filteroptions, *choices)
                om1.grid(row=i, column=1)
                # my_w.mainloop()  # Keep the window open
                i += 1

                # #             "Observing Notes": "Weather, seeing was nice.",
                obsnotes_label = tk.Label(root, text="Observing Notes", justify=tk.LEFT)
                obsnotes_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                obsnotes_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                obsnotes_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1


                # root = tk.Tk()
                # root.title(f"EXOTIC v{__version__}")
                #
                # window_label = tk.Label(root,
                #                         text="""Please enter the following information:""",
                #                         font=("Helvetica 15 bold"),
                #                         justify=tk.LEFT,
                #                         padx=20)  # .pack()
                # window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                # i, j = 1, 0
                platesolve = tk.BooleanVar()
                platesolve_check = tk.Checkbutton(root, text='Plate solve my images', variable=platesolve, onvalue=True,
                                                  offvalue=False)
                platesolve_check.grid(row=i, column=j, sticky=tk.W, pady=2)
                i += 1

                alignment = tk.BooleanVar()
                alignment_check = tk.Checkbutton(root, text='Align my images', variable=alignment, onvalue=1, offvalue=0)
                alignment_check.grid(row=i, column=j, sticky=tk.W, pady=2)
                i += 1

                targetpos_label = tk.Label(root, text="Target Star X & Y Pixel Position", justify=tk.LEFT)
                targetpos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                targetpos_entry.insert(tk.END, "[x, y]")
                targetpos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                targetpos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                comppos_label = tk.Label(root, text="Comparison Star(s) X & Y Pixel Position", justify=tk.LEFT)
                comppos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                comppos_entry.insert(tk.END, "[x1, y1], [x2, y2]")
                comppos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                comppos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                # # Button for closing
                # exit_button = tk.Button(root, text="Next", command=root.destroy)
                # exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)
                # # exit_button.pack(pady=20)
                # root.update()
                # root.mainloop()

                def save_input():
                    input_data['elevation'] = float(elevation_entry.get())
                    input_data['comppos'] = ast.literal_eval(comppos_entry.get())
                    input_data['targetpos'] = ast.literal_eval(targetpos_entry.get())
                    input_data['obsnotes'] = obsnotes_entry.get()
                    input_data['obsfilter'] = filteroptions.get()
                    input_data['pixscale'] = pixscale_entry.get()
                    input_data['pixbin'] = pixbin_entry.get()
                    input_data['cameratype'] = cameratype_entry.get()
                    input_data['obscode'] = obscode_entry.get()
                    input_data['obsdate'] = obsdate_entry.get()
                    input_data['lat'] = lat_entry.get()
                    input_data['long'] = long_entry.get()
                    input_data['secondobscode'] = secondobscode_entry.get()

                    if platesolve.get() == True:
                        input_data['platesolve'] = 'y'
                    else:
                        input_data['platesolve'] = 'n'

                    if platesolve.get() == True:
                        input_data['alignment'] = 'y'
                    else:
                        input_data['alignment'] = 'n'

                    root.destroy()

                # Button for closing
                exit_button = tk.Button(root, text="Next", command=save_input)
                # exit_button.pack(pady=20)
                exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)

                tk.mainloop()
            else:
                # raise(Exception("feature not supported yet"))
                pass

            try:
                if filteroptions.get() == "N/A":
                    root = tk.Tk()
                    root.title(f"EXOTIC v{__version__}")

                    i = 0; j = 0

                    tk.Label(root,
                             text="Please enter the following information for your custom filter:",
                             font=("Helvetica 15 bold"),
                             justify=tk.LEFT,
                             padx=20).grid(row=i, column=j, sticky=tk.W, pady=2)
                    i += 1

                    # #
                    # "Filter Minimum Wavelength (nm)": null,
                    filtermin_label = tk.Label(root, text="Filter Minimum Wavelength (nm)", justify=tk.LEFT)
                    filtermin_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                    filtermin_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                    filtermin_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                    i += 1

                    # "Filter Maximum Wavelength (nm)": null
                    filtermax_label = tk.Label(root, text="Filter Maximum Wavelength (nm)", justify=tk.LEFT)
                    filtermax_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                    filtermax_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                    filtermax_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                    i += 1

                    def save_input():
                        input_data['filtermax'] = float(filtermax_entry.get())
                        input_data['filtermin'] = float(filtermin_entry.get())
                        root.destroy()

                    # Button for closing
                    exit_button = tk.Button(root, text="Next", command=save_input)
                    # exit_button.pack(pady=20)
                    exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)

                    root.mainloop()
            except:
                pass

            # then ask user how they want to enter the planetary information
            root = tk.Tk()
            root.title(f"EXOTIC v{__version__}")

            planetparams = tk.StringVar()

            tk.Label(root,
                     text="""How would you like to input your planetary system parameters?""",
                     font=("Helvetica 15 bold"),
                     justify=tk.LEFT,
                     padx=20).pack()

            tk.Radiobutton(root,
                           text="Manually",
                           padx=20,
                           variable=planetparams,
                           value="manual").pack(anchor=tk.W)

            tk.Radiobutton(root,
                           text="Automatically adopt all planetary parameters\nfrom the NASA Exoplanet Archive",
                           padx=20,
                           variable=planetparams,
                           value="nea").pack(anchor=tk.W)

            tk.Radiobutton(root,
                           text="From a pre-existing input file (e.g., inits.json)",
                           padx=20,
                           variable=planetparams,
                           value="inits").pack(anchor=tk.W)

            # Button for closing
            exit_button = tk.Button(root, text="Next", command=root.destroy)
            exit_button.pack(pady=20, anchor=tk.E)

            root.mainloop()

            if planetparams.get() == "manual":
                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                initparams = tk.IntVar()

                window_label = tk.Label(root,
                                        text="""Please enter the following information:""",
                                        font=("Helvetica 15 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                # Set up rows + columns
                i = 1;
                j = 0

                #         "Planet Name": "HAT-P-32 b",
                planet_label = tk.Label(root, text="Planet Name", justify=tk.LEFT)
                planet_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                planet_entry.insert(tk.END, "HAT-P-32 b")
                planet_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                planet_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Host Star Name": "HAT-P-32",
                star_label = tk.Label(root, text="Host Star Name", justify=tk.LEFT)
                star_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                star_entry.insert(tk.END, "HAT-P-32")
                star_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                star_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Target Star RA": "02:04:10",
                targetRA_label = tk.Label(root, text="Host Star Right Ascension", justify=tk.LEFT)
                targetRA_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                targetRA_entry.insert(tk.END, "02:04:10")
                targetRA_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                targetRA_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Target Star Dec": "+46:41:23",
                targetDEC_label = tk.Label(root, text="Host Star Declination", justify=tk.LEFT)
                targetDEC_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                targetDEC_entry.insert(tk.END, "+46:41:23")
                targetDEC_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                targetDEC_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Orbital Period (days)": 2.1500082,
                period_label = tk.Label(root, text="Planet's orbital period (days)", justify=tk.LEFT)
                period_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                period_entry.insert(tk.END, "2.1500082")
                period_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                period_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Orbital Period Uncertainty": 1.3e-07,
                perioderr_label = tk.Label(root, text="Planet's orbital period uncertainty (days)", justify=tk.LEFT)
                perioderr_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                perioderr_entry.insert(tk.END, "1.3e-07")
                perioderr_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                perioderr_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Published Mid-Transit Time (BJD-UTC)": 2455867.402743,
                Tmid_label = tk.Label(root, text="Published Mid-Transit Time (BJD-UTC)", justify=tk.LEFT)
                Tmid_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                Tmid_entry.insert(tk.END, "2455867.402743")
                Tmid_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                Tmid_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Mid-Transit Time Uncertainty": 4.9e-05,
                Tmiderr_label = tk.Label(root, text="Mid-Transit Time Uncertainty", justify=tk.LEFT)
                Tmiderr_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                Tmiderr_entry.insert(tk.END, "4.9e-05")
                Tmiderr_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                Tmiderr_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Ratio of Planet to Stellar Radius (Rp/Rs)": 0.14886235252742716,
                rprs_label = tk.Label(root, text="Ratio of Planet to Stellar Radius (Rp/Rs)", justify=tk.LEFT)
                rprs_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                rprs_entry.insert(tk.END, "0.14886235252742716")
                rprs_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                rprs_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Ratio of Planet to Stellar Radius (Rp/Rs) Uncertainty": 0.0005539487393037134,
                rprserr_label = tk.Label(root, text="Ratio of Planet to Stellar Radius (Rp/Rs) Uncertainty", justify=tk.LEFT)
                rprserr_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                rprserr_entry.insert(tk.END, "0.0005539487393037134")
                rprserr_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                rprserr_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Ratio of Distance to Stellar Radius (a/Rs)": 5.344,
                aRs_label = tk.Label(root, text="Ratio of Distance to Stellar Radius (a/Rs)", justify=tk.LEFT)
                aRs_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                aRs_entry.insert(tk.END, "5.344")
                aRs_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                aRs_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Ratio of Distance to Stellar Radius (a/Rs) Uncertainty": 0.039496835316262996,
                aRserr_label = tk.Label(root, text="Ratio of Distance to Stellar Radius (a/Rs) Uncertainty", justify=tk.LEFT)
                aRserr_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                aRserr_entry.insert(tk.END, "0.039496835316262996")
                aRserr_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                aRserr_entry.grid(row=i, column=j + 1,  sticky=tk.W, pady=2)
                i += 1

                #         "Orbital Inclination (deg)": 88.98,
                inc_label = tk.Label(root, text="Orbital Inclination (degrees)", justify=tk.LEFT)
                inc_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                inc_entry.insert(tk.END, "88.98")
                inc_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                inc_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Orbital Inclination (deg) Uncertainty": 0.7602631123499285,
                incerr_label = tk.Label(root, text="Orbital Inclination (degrees) Uncertainty", justify=tk.LEFT)
                incerr_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                incerr_entry.insert(tk.END, "0.7602631123499285")
                incerr_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                incerr_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Orbital Eccentricity (0 if null)": 0.159,
                ecc_label = tk.Label(root, text="Orbital Eccentricity (0 if null)", justify=tk.LEFT)
                ecc_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                ecc_entry.insert(tk.END, "0.159")
                ecc_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                ecc_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Effective Temperature (K)": 6001.0,
                Teff_label = tk.Label(root, text="Star Effective Temperature (K)", justify=tk.LEFT)
                Teff_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                Teff_entry.insert(tk.END, "6001.0")
                Teff_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                Teff_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Effective Temperature (+) Uncertainty": 88.0,
                Tefferrpos_label = tk.Label(root, text="Star Effective Temperature Positive (+) Uncertainty", justify=tk.LEFT)
                Tefferrpos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                Tefferrpos_entry.insert(tk.END, "+88.0")
                Tefferrpos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                Tefferrpos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Effective Temperature (-) Uncertainty": -88.0,
                Tefferrneg_label = tk.Label(root, text="Star Effective Temperature Negative (-) Uncertainty", justify=tk.LEFT)
                Tefferrneg_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                Tefferrneg_entry.insert(tk.END, "-88.0")
                Tefferrneg_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                Tefferrneg_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Metallicity ([FE/H])": -0.16,
                FeH_label = tk.Label(root, text="Star Metallicity ([Fe/H])", justify=tk.LEFT)
                FeH_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                FeH_entry.insert(tk.END, "-0.1")
                FeH_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                FeH_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Metallicity (+) Uncertainty": 0.08,
                FeHerrpos_label = tk.Label(root, text="Star Metallicity Positive (+) Uncertainty", justify=tk.LEFT)
                FeHerrpos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                FeHerrpos_entry.insert(tk.END, "0.08")
                FeHerrpos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                FeHerrpos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Metallicity (-) Uncertainty": -0.08,
                FeHerrneg_label = tk.Label(root, text="Star Metallicity Negative (-) Uncertainty", justify=tk.LEFT)
                FeHerrneg_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                FeHerrneg_entry.insert(tk.END, "-0.08")
                FeHerrneg_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                FeHerrneg_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Surface Gravity (log(g))": 4.22,
                logg_label = tk.Label(root, text="Star Surface Gravity (log(g))", justify=tk.LEFT)
                logg_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                logg_entry.insert(tk.END, "4.22")
                logg_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                logg_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Surface Gravity (+) Uncertainty": 0.04,
                loggerrpos_label = tk.Label(root, text="Star Surface Gravity Positive (+) Uncertainty", justify=tk.LEFT)
                loggerrpos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                loggerrpos_entry.insert(tk.END, "0.1")
                loggerrpos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                loggerrpos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Star Surface Gravity (-) Uncertainty": -0.04
                loggerrneg_label = tk.Label(root, text="Star Surface Gravity Negative (-) Uncertainty", justify=tk.LEFT)
                loggerrneg_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                loggerrneg_entry.insert(tk.END, "-0.1")
                loggerrneg_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                loggerrneg_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                def save_input():
                    input_data['inc'] = inc_entry.get()
                    input_data['aRserr'] = aRserr_entry.get()
                    input_data['aRs'] = aRs_entry.get()
                    input_data['rprserr'] = rprserr_entry.get()
                    input_data['rprs'] = rprs_entry.get()
                    input_data['Tmiderr'] = Tmiderr_entry.get()
                    input_data['Tmid'] = Tmid_entry.get()
                    input_data['perioderr'] = perioderr_entry.get()
                    input_data['period'] = period_entry.get()
                    input_data['targetDEC'] = targetDEC_entry.get()
                    input_data['targetRA'] = targetRA_entry.get()
                    input_data['star'] = star_entry.get()
                    input_data['planet'] = planet_entry.get()
                    input_data['loggerrneg'] = loggerrneg_entry.get()
                    input_data['loggerrpos'] = loggerrpos_entry.get()
                    input_data['logg'] = logg_entry.get()
                    input_data['FeHerrneg'] = FeHerrneg_entry.get()
                    input_data['FeHerrpos'] = FeHerrpos_entry.get()
                    input_data['Tefferrneg'] = Tefferrneg_entry.get()
                    input_data['Teff'] = Teff_entry.get()
                    input_data['Tefferrpos'] = Tefferrpos_entry.get()
                    input_data['FeH'] = FeH_entry.get()
                    input_data['ecc'] = ecc_entry.get()
                    input_data['incerr'] = incerr_entry.get()
                    root.destroy()

                # Button for closing
                exit_button = tk.Button(root, text="Next", command=save_input)
                # exit_button.pack(pady=20)
                exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)

                tk.mainloop()

            elif planetparams.get() == 'nea':
                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                initparams = tk.IntVar()

                window_label = tk.Label(root,
                                        text="""Please enter the following information:""",
                                        font=("Helvetica 15 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                # Set up rows + columns
                i = 1;
                j = 0

                #         "Planet Name": "HAT-P-32 b",
                planet_label = tk.Label(root, text="Planet Name", justify=tk.LEFT)
                planet_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                planet_entry.insert(tk.END, "HAT-P-32 b")
                planet_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                planet_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                #         "Host Star Name": "HAT-P-32",
                star_label = tk.Label(root, text="Host Star Name", justify=tk.LEFT)
                star_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
                star_entry.insert(tk.END, "HAT-P-32")
                star_label.grid(row=i, column=j, sticky=tk.W, pady=2)
                star_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
                i += 1

                def save_input():
                    input_data['star'] = star_entry.get()
                    input_data['planet'] = planet_entry.get()
                    root.destroy()

                # Button for closing
                exit_button = tk.Button(root, text="Next", command=save_input)
                # exit_button.pack(pady=20)
                exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)

                tk.mainloop()


            if (planetparams.get() == "inits") or (obsinfo.get() == "inits"):
                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                # # "Directory with FITS files": "sample-data/HatP32Dec202017",
                inits_dir = FileSelect(root, "Please select your initialization file")
                inits_dir.grid(row=0)

                def save_inputs():
                    input_data['inits_dir'] = inits_dir.file_path
                    root.destroy()

                # Button for closing
                exit_button = tk.Button(root, text="Next", command=save_inputs)
                # exit_button.pack(pady=20)
                exit_button.grid(row=1, column=3, sticky=tk.W, pady=10)

                tk.mainloop()

            # if (obsinfo.get() == "manual"):
            #     root = tk.Tk()
            #     root.title(f"EXOTIC v{__version__}")
            #
            #     window_label = tk.Label(root,
            #                             text="""Please enter the following information:""",
            #                             font=("Helvetica 15 bold"),
            #                             justify=tk.LEFT,
            #                             padx=20)  # .pack()
            #     window_label.grid(row=0, column=0, sticky=tk.N, pady=6)
            #
            #     i, j = 1, 0
            #     platesolve = tk.BooleanVar()
            #     platesolve_check = tk.Checkbutton(root, text='Plate solve my images', variable=platesolve, onvalue=True,
            #                                       offvalue=False)
            #     platesolve_check.grid(row=i, column=j, sticky=tk.W, pady=2)
            #     i += 1
            #
            #     alignment = tk.BooleanVar()
            #     alignment_check = tk.Checkbutton(root, text='Align my images', variable=alignment, onvalue=1, offvalue=0)
            #     alignment_check.grid(row=i, column=j, sticky=tk.W, pady=2)
            #     i += 1
            #
            #     targetpos_label = tk.Label(root, text="Target Star X & Y Pixel Position", justify=tk.LEFT)
            #     targetpos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
            #     targetpos_entry.insert(tk.END, "[x, y]")
            #     targetpos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
            #     targetpos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
            #     targetpos = targetpos_entry.get()
            #     i += 1
            #
            #     comppos_label = tk.Label(root, text="Comparison Star(s) X & Y Pixel Position", justify=tk.LEFT)
            #     comppos_entry = tk.Entry(root, font=("Helvetica 12"), justify=tk.LEFT)
            #     comppos_entry.insert(tk.END, "[x1, y1], [x2, y2]")
            #     comppos_label.grid(row=i, column=j, sticky=tk.W, pady=2)
            #     comppos_entry.grid(row=i, column=j + 1, sticky=tk.W, pady=2)
            #     comppos = comppos_entry.get()
            #     i += 1
            #
            #     # Button for closing
            #     exit_button = tk.Button(root, text="Next", command=root.destroy)
            #     exit_button.grid(row=i, column=3, sticky=tk.W, pady=10)
            #     # exit_button.pack(pady=20)
            #     root.update()
            #     root.mainloop()


            # root.mainloop()

            # Create an initialization file if it does not already exist
            if (planetparams.get() != "inits") or (obsinfo.get() != "inits"):
                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                initparams = tk.IntVar()

                window_label = tk.Label(root,
                                        text="To make analyzing these data in the future more easy, EXOTIC will create an initialization file for you.",
                                        font=("Helvetica 14 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                # Set up rows + columns
                i = 1;
                j = 0

                folderPath = tk.StringVar()

                # # "Directory with FITS files": "sample-data/HatP32Dec202017",
                initssave_dir = FolderSelect(root, "Folder to save your initialization file:")
                initssave_dir.grid(row=i)
                i += 1

                def save_input():
                    input_data['initssave_dir'] = initssave_dir.folderPath.get()
                    root.destroy()

                # Button for closing
                exit_button = tk.Button(root, text="Next", command=save_input)
                # exit_button.pack(pady=20)
                exit_button.grid(row=1, column=3, sticky=tk.W, pady=10)

                tk.mainloop()


                #create the inits file here
                new_inits = {'inits_guide':{}, 'user_info':{}, 'planetary_parameters':{}, 'optional_info':{}}
                new_inits['inits_guide'] = {
                    "Title": "EXOTIC's Initialization File",
                    "Comment": "Please answer all the following requirements below by following the format of the given",
                    "Comment1": "sample dataset HAT-P-32 b. Edit this file as needed to match the data wanting to be reduced.",
                    "Comment2": "Do not delete areas where there are quotation marks, commas, and brackets.",
                    "Comment3": "The inits_guide dictionary (these lines of text) does not have to be edited",
                    "Comment4": "and is only here to serve as a guide. Will be updated per user's advice.",
                    "Image Calibrations Directory Guide": "Enter in the path to image calibrations or enter in null for none.",
                    "Planetary Parameters Guide": "For planetary parameters that are not filled in, enter in null.",
                    "Comparison Star(s) Guide": "Up to 10 comparison stars can be added following the format given below.",
                    "Obs. Latitude Guide": "Indicate the sign (+ North, - South) before the degrees. Needs to be in decimal or HH:MM:SS format.",
                    "Obs. Longitude Guide": "Indicate the sign (+ East, - West) before the degrees. Needs to be in decimal or HH:MM:SS format.",
                    "Plate Solution": "For your image to be given a plate solution, type y.",
                    "Plate Solution Disclaimer": "One of your imaging files will be publicly viewable on nova.astrometry.net.",
                    "Standard Filter": "To use EXOTIC standard filters, type only the filter name.",
                    "Custom Filter": "To use a custom filter, enter in the FWHM in optional_info.",
                    "Target Star RA": "Must be in HH:MM:SS sexagesimal format.",
                    "Target Star DEC": "Must be in +/-DD:MM:SS sexagesimal format with correct sign at the beginning (+ or -).",
                    "Formatting of null": "Due to the file being a .json, null is case sensitive and must be spelled as shown.",
                    "Decimal Format": "Leading zero must be included when appropriate (Ex: 0.32, .32 or 00.32 causes errors.)."
                }

                null = None

                if obsinfo.get() == 'manual':
                    new_inits['user_info'] = {
                        "Directory with FITS files": FITS_dir.folder_path,
                        "Directory to Save Plots": save_dir.folder_path,
                        # "Directory of Flats": flats_dir.folder_path,
                        # "Directory of Darks": darks_dir.folder_path,
                        # "Directory of Biases": biases_dir.folder_path,

                        "AAVSO Observer Code (N/A if none)": input_data['obscode'],
                        "Secondary Observer Codes (N/A if none)": input_data['secondobscode'],

                        "Observation date": input_data['obsdate'],
                        "Obs. Latitude": input_data['lat'],
                        "Obs. Longitude": input_data['long'],
                        "Obs. Elevation (meters)": float(input_data.get('elevation',0)),
                        "Camera Type (CCD or DSLR)": input_data['cameratype'],
                        "Pixel Binning": input_data['pixbin'],
                        "Filter Name (aavso.org/filters)": input_data['obsfilter'],
                        "Observing Notes": input_data['obsnotes'],

                        "Plate Solution? (y/n)": input_data['platesolve'],
                        "Align Images? (y/n)": input_data['alignment'],

                        "Target Star X & Y Pixel": (input_data['targetpos']),
                        "Comparison Star(s) X & Y Pixel": (input_data['comppos'])
                    }

                    if flats_dir.folder_path == 'null':
                        new_inits['user_info']["Directory of Flats"] = null
                    else:
                        new_inits['user_info']["Directory of Flats"] = flats_dir.folder_path

                    if darks_dir.folder_path == 'null':
                        new_inits['user_info']["Directory of Darks"] = null
                    else:
                        new_inits['user_info']["Directory of Darks"] = darks_dir.folder_path

                    if biases_dir.folder_path == 'null':
                        new_inits['user_info']["Directory of Biases"] = null
                    else:
                        new_inits['user_info']["Directory of Biases"] = biases_dir.folder_path

                elif obsinfo.get() == 'inits':
                    with open(input_data['inits_dir'], "r") as confirmed:
                        original_inits = json.load(confirmed)

                    new_inits['user_info'] = original_inits['user_info']

                if (planetparams.get() == "manual"):
                    new_inits['planetary_parameters'] = {
                        "Target Star RA": input_data['targetRA'],
                        "Target Star Dec": input_data['targetDEC'],
                        "Planet Name": input_data['planet'],
                        "Host Star Name": input_data['star'],
                        "Orbital Period (days)": float(input_data['period']),
                        "Orbital Period Uncertainty": float(input_data['perioderr']),
                        "Published Mid-Transit Time (BJD-UTC)": float(input_data['Tmid']),
                        "Mid-Transit Time Uncertainty": float(input_data['Tmiderr']),
                        "Ratio of Planet to Stellar Radius (Rp/Rs)": float(input_data['rprs']),
                        "Ratio of Planet to Stellar Radius (Rp/Rs) Uncertainty": float(input_data['rprserr']),
                        "Ratio of Distance to Stellar Radius (a/Rs)": float(input_data['aRs']),
                        "Ratio of Distance to Stellar Radius (a/Rs) Uncertainty": float(input_data['aRserr']),
                        "Orbital Inclination (deg)": float(input_data['inc']),
                        "Orbital Inclination (deg) Uncertainty": float(input_data['incerr']),
                        "Orbital Eccentricity (0 if null)": float(input_data['ecc']),
                        "Star Effective Temperature (K)": float(input_data['Teff']),
                        "Star Effective Temperature (+) Uncertainty": float(input_data['Tefferrpos']),
                        "Star Effective Temperature (-) Uncertainty": float(input_data['Tefferrneg']),
                        "Star Metallicity ([FE/H])": float(input_data['FeH']),
                        "Star Metallicity (+) Uncertainty": float(input_data['FeHerrpos']),
                        "Star Metallicity (-) Uncertainty": float(input_data['FeHerrneg']),
                        "Star Surface Gravity (log(g))": float(input_data['logg']),
                        "Star Surface Gravity (+) Uncertainty": float(input_data['loggerrpos']),
                        "Star Surface Gravity (-) Uncertainty": float(input_data['loggerrneg'])
                    }
                elif (planetparams.get() == "inits"):
                    with open(input_data['inits_dir'], "r") as confirmed:
                        original_inits = json.load(confirmed)

                    new_inits['planetary_parameters'] = original_inits['planetary_parameters']

                else:
                    # Just put in dummy values as they will be overwritten by the NEA later
                    new_inits['planetary_parameters'] = {
                        "Target Star RA": "00:00:00",
                        "Target Star Dec": "+00:00:00",
                        "Planet Name":  input_data['planet'],
                        "Host Star Name":  input_data['star'],
                        "Orbital Period (days)": 0.,
                        "Orbital Period Uncertainty": 0.,
                        "Published Mid-Transit Time (BJD-UTC)": 0.,
                        "Mid-Transit Time Uncertainty": 0.,
                        "Ratio of Planet to Stellar Radius (Rp/Rs)": 0.,
                        "Ratio of Planet to Stellar Radius (Rp/Rs) Uncertainty": 0.,
                        "Ratio of Distance to Stellar Radius (a/Rs)": 0.,
                        "Ratio of Distance to Stellar Radius (a/Rs) Uncertainty": 0.,
                        "Orbital Inclination (deg)": 0.,
                        "Orbital Inclination (deg) Uncertainty": 0.,
                        "Orbital Eccentricity (0 if null)": 0.,
                        "Star Effective Temperature (K)": 0.,
                        "Star Effective Temperature (+) Uncertainty": 0.,
                        "Star Effective Temperature (-) Uncertainty": 0.,
                        "Star Metallicity ([FE/H])": 0.,
                        "Star Metallicity (+) Uncertainty": 0.,
                        "Star Metallicity (-) Uncertainty": 0.,
                        "Star Surface Gravity (log(g))": 0.,
                        "Star Surface Gravity (+) Uncertainty": 0.,
                        "Star Surface Gravity (-) Uncertainty": 0.
                    }

                null = None
                new_inits['optional_info'] = {
                    "Filter Minimum Wavelength (nm)": input_data.get('filtermin',null),
                    "Filter Maximum Wavelength (nm)": input_data.get('filtermax',null)
                }

                if 'pixscale' not in input_data.keys():
                    new_inits['optional_info']['Pixel Scale (Ex: 5.21 arcsecs/pixel)'] = null
                elif input_data['pixscale'] == 'null':
                    new_inits['optional_info']['Pixel Scale (Ex: 5.21 arcsecs/pixel)'] = null
                else:
                    new_inits['optional_info']['Pixel Scale (Ex: 5.21 arcsecs/pixel)'] = input_data['pixscale']

                now = datetime.now()
                dt_string = now.strftime("%d_%m_%Y__%H_%M_%S")
                fname = os.path.join(input_data['initssave_dir'], "inits_"+dt_string+".json")
                with open(fname, "w") as initsf:
                    json.dump(new_inits, initsf, indent=4)
                print(f"{fname} saved!")

                root = tk.Tk()
                root.title(f"EXOTIC v{__version__}")

                window_label = tk.Label(root,
                                        text="Your initialization file has been created!",
                                        font=("Helvetica 14 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=0, column=0, sticky=tk.N, pady=6)

                window_label = tk.Label(root,
                                        text=f"{fname}",
                                        font=("Helvetica 14 bold"),
                                        justify=tk.LEFT,
                                        padx=20)  # .pack()
                window_label.grid(row=1, column=0, sticky=tk.N, pady=6)

                # Button for closing
                exit_button = tk.Button(root, text="Run EXOTIC", command=root.destroy)
                # exit_button.pack(pady=20)
                exit_button.grid(row=2, column=3, sticky=tk.W, pady=10)

                tk.mainloop()

            #         If the user already has an inits file, then go for it
            try:
                if planetparams.get() == 'inits':
                    try:
                        subprocess.run(['exotic', '--reduce', inits_dir.file_path, '-ov'], check=True)
                    except:
                        try:
                            subprocess.run(['python3', 'exotic.py', '--reduce', inits_dir.file_path, '-ov'], check=True)
                        except:
                            subprocess.run(['python3', 'exotic/exotic.py', '--reduce', inits_dir.file_path, '-ov'], check=True)

                elif planetparams.get() == 'nea':
                    try:
                        subprocess.run(['exotic', '--reduce', fname, '-nea'], check=True)
                    except:
                        try:
                            subprocess.run(['python3', 'exotic.py', '--reduce', fname, '-nea'], check=True)
                        except:
                            subprocess.run(['python3', 'exotic/exotic.py', '--reduce', fname, '-nea'], check=True)

                else:
                    try:
                        subprocess.run(['exotic', '--reduce', fname], check=True)
                    except:
                        try:
                            subprocess.run(['python3', 'exotic.py', '--reduce', fname], check=True)
                        except:
                            subprocess.run(['python3', 'exotic/exotic.py', '--reduce', fname], check=True)
            except:
                print("\n\n################################################")
                print("ERROR: Please contact the Exoplanet Watch Team for help on our Slack Workspace in the #data-reduction channel!")
                print("You can sign up for our free Slack Workspace here: https://join.slack.com/t/uol-ets/shared_invite/zt-mvb4ljbo-LRBgpk3uMmUokbs4ge2JlA")
                print("################################################\n\n")
                pass

if __name__ == "__main__":
    main()


# ########################################################################### #
#    Copyright (c) 2019-2020, California Institute of Technology.
#    All rights reserved.  Based on Government Sponsored Research under
#    contracts NNN12AA01C, NAS7-1407 and/or NAS7-03001.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#      1. Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#      2. Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer in
#         the documentation and/or other materials provided with the
#         distribution.
#      3. Neither the name of the California Institute of
#         Technology (Caltech), its operating division the Jet Propulsion
#         Laboratory (JPL), the National Aeronautics and Space
#         Administration (NASA), nor the names of its contributors may be
#         used to endorse or promote products derived from this software
#         without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CALIFORNIA
#    INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#    TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ########################################################################### #
#    EXOplanet Transit Interpretation Code (EXOTIC)
#    # NOTE: See companion file version.py for version info.
# ########################################################################### #

import logging
import numpy as np
import lmfit as lm
import matplotlib.pyplot as plt
import ldtk
from ldtk import LDPSetCreator, BoxcarFilter
from ldtk.ldmodel import LinearModel, QuadraticModel, NonlinearModel

log = logging.getLogger(__name__)


class LDPSet(ldtk.LDPSet):
    """
    A. NIESSNER: INLINE HACK TO ldtk.LDPSet
    """
    @staticmethod
    def is_mime(): return True

    @property
    def profile_mu(self): return self._mu
    pass


setattr(ldtk, 'LDPSet', LDPSet)
setattr(ldtk.ldtk, 'LDPSet', LDPSet)


def createldgrid(minmu, maxmu, orbp,
                 ldmodel='nonlinear', phoenixmin=1e-1,
                 segmentation=int(10), verbose=False):
    """
    G. ROUDIER: Wrapper around LDTK downloading tools
    LDTK: Parviainen et al. https://github.com/hpparvi/ldtk
    """
    tstar = orbp['T*']
    terr = np.sqrt(abs(orbp['T*_uperr']*orbp['T*_lowerr']))
    fehstar = orbp['FEH*']
    feherr = np.sqrt(abs(orbp['FEH*_uperr']*orbp['FEH*_lowerr']))
    loggstar = orbp['LOGG*']
    loggerr = np.sqrt(abs(orbp['LOGG*_uperr']*orbp['LOGG*_lowerr']))
    log.warning(f">-- Temperature: {tstar} +/- {terr}")
    log.warning(f">-- Metallicity: {fehstar} +/- {feherr}")
    log.warning(f">-- Surface Gravity: {loggstar} +/- {loggerr}")
    niter = int(len(minmu)/segmentation) + 1
    allcl = None
    allel = None
    out = {}
    avmu = [np.mean([mm, xm]) for mm, xm in zip(minmu, maxmu)]
    for i in np.arange(niter):
        loweri = i*segmentation
        upperi = (i+1)*segmentation
        if i == (niter-1):
            upperi = len(avmu)
        munm = 1e3*np.array(avmu[loweri:upperi])
        munmmin = 1e3*np.array(minmu[loweri:upperi])
        munmmax = 1e3*np.array(maxmu[loweri:upperi])
        filters = [BoxcarFilter(str(mue), mun, mux)
                   for mue, mun, mux in zip(munm, munmmin, munmmax)]
        sc = LDPSetCreator(teff=(tstar, terr), logg=(loggstar, loggerr),
                           z=(fehstar, feherr), filters=filters)
        ps = sc.create_profiles(nsamples=int(1e4))
        itpfail = False
        for testprof in ps.profile_averages:
            if np.all(~np.isfinite(testprof)):
                itpfail = True
            pass
        nfail = 1e0
        while itpfail:
            nfail *= 2
            sc = LDPSetCreator(teff=(tstar, nfail*terr), logg=(loggstar, loggerr),
                               z=(fehstar, feherr), filters=filters)
            ps = sc.create_profiles(nsamples=int(1e4))
            itpfail = False
            for testprof in ps.profile_averages:
                if np.all(~np.isfinite(testprof)):
                    itpfail = True
                pass
            pass
        cl, el = ldx(ps.profile_mu, ps.profile_averages, ps.profile_uncertainties,
                     mumin=phoenixmin, debug=verbose, model=ldmodel)
        if allcl is None:
            allcl = cl
        else:
            allcl = np.concatenate((allcl, cl), axis=0)
        if allel is None:
            allel = el
        else:
            allel = np.concatenate((allel, el), axis=0)
        pass
    allel[allel > 1.] = 0.
    allel[~np.isfinite(allel)] = 0.
    out['MU'] = avmu
    out['LD'] = allcl.T
    out['ERR'] = allel.T
    for i in range(0, len(allcl.T)):
        log.warning(f">-- LD{int(i)}: {float(allcl.T[i])} +/- {float(allel.T[i])}")
        pass
    return out


def ldx(psmu, psmean, psstd, mumin=1e-1, debug=False, model='nonlinear'):
    """
    G. ROUDIER: Limb darkening coefficient retrievial on PHOENIX GRID models,
    OPTIONAL mumin set up on HAT-P-41 stellar class
    """
    mup = np.array(psmu).copy()
    prfs = np.array(psmean).copy()
    sprfs = np.array(psstd).copy()
    nwave = prfs.shape[0]
    select = (mup > mumin)
    fitmup = mup[select]
    fitprfs = prfs[:, select]
    fitsprfs = sprfs[:, select]
    cl = []
    el = []
    params = lm.Parameters()
    params.add('gamma1', value=1e-1)
    params.add('gamma2', value=5e-1)
    params.add('gamma3', value=1e-1)
    params.add('gamma4', expr='1 - gamma1 - gamma2 - gamma3')

    if debug:
        plt.figure()
    for iwave in np.arange(nwave):
        select = fitsprfs[iwave] == 0e0
        if True in select:
            fitsprfs[iwave][select] = 1e-10
        if model == 'linear':
            params['gamma1'].value = 0
            params['gamma1'].vary = False
            params['gamma3'].value = 0
            params['gamma3'].vary = False
            params['gamma4'].value = 0
            params['gamma4'].vary = False
            out = lm.minimize(ln_ldx, params,
                              args=(fitmup, fitprfs[iwave], fitsprfs[iwave]))
            cl.append([out.params['gamma1'].value])
            el.append([out.params['gamma1'].stderr])
            pass
        if model == 'quadratic':
            params['gamma1'].value = 0
            params['gamma1'].vary = False
            params['gamma3'].value = 0
            params['gamma3'].vary = False
            out = lm.minimize(qd_ldx, params,
                              args=(mup, prfs[iwave], sprfs[iwave]))
            cl.append([out.params['gamma2'].value, out.params['gamma4'].value])
            el.append([out.params['gamma2'].stderr, out.params['gamma4'].stderr])
            pass
        if model == 'nonlinear':
            out = lm.minimize(nl_ldx, params,
                              args=(fitmup, fitprfs[iwave], fitsprfs[iwave]))
            cl.append([out.params['gamma1'].value, out.params['gamma2'].value,
                       out.params['gamma3'].value, out.params['gamma4'].value])
            el.append([out.params['gamma1'].stderr, out.params['gamma2'].stderr,
                       out.params['gamma3'].stderr, out.params['gamma4'].stderr])
            pass
        if debug:
            plt.plot(mup, prfs[iwave], 'k^')
            plt.errorbar(fitmup, fitprfs[iwave], yerr=fitsprfs[iwave], ls='None')
            if model == 'linear':
                plt.plot(fitmup, ln_ldx(out.params, fitmup))
            if model == 'quadratic':
                plt.plot(fitmup, qd_ldx(out.params, fitmup))
            if model == 'nonlinear':
                plt.plot(fitmup, nl_ldx(out.params, fitmup))
            pass
        pass
    if debug:
        plt.ylabel('$I(\\mu)$')
        plt.xlabel('$\\mu$')
        plt.title(model)
        plt.show()
        pass
    return np.array(cl), np.array(el)


def ln_ldx(params, x, data=None, weights=None):
    """
    G. ROUDIER: Linear law
    """
    gamma1 = params['gamma1'].value
    model = LinearModel.evaluate(x, np.array(gamma1))
    if data is None:
        return model
    if weights is None:
        return data - model
    return (data - model)/weights


def qd_ldx(params, x, data=None, weights=None):
    """
    G. ROUDIER: Quadratic law
    """
    gamma1 = params['gamma2'].value
    gamma2 = params['gamma4'].value
    model = QuadraticModel.evaluate(x, np.array((gamma1, gamma2)))
    if data is None:
        return model
    if weights is None:
        return data - model
    return (data - model)/weights


def nl_ldx(params, x, data=None, weights=None):
    """
    G. ROUDIER: Non Linear law
    """
    gamma1 = params['gamma1'].value
    gamma2 = params['gamma2'].value
    gamma3 = params['gamma3'].value
    gamma4 = params['gamma4'].value
    model = NonlinearModel.evaluate(x, np.array((gamma1, gamma2, gamma3, gamma4)))
    if data is None:
        return model
    if weights is None:
        return data - model
    return (data - model)/weights




import logging
import json
import sys
from pathlib import Path

try:
    from util import user_input, dms_to_dd, open_elevation, typecast_check, init_params
except ImportError:
    from .util import user_input, dms_to_dd, open_elevation, typecast_check, init_params


log = logging.getLogger(__name__)
logging.basicConfig(filename='exotic.log', level=logging.DEBUG)
consoleFormatter = logging.Formatter("%(message)s")
consoleHandler = logging.StreamHandler(sys.stdout)
consoleHandler.setFormatter(consoleFormatter)
consoleHandler.setLevel(logging.INFO)
log.addHandler(consoleHandler)


class Inputs:

    def __init__(self, init_opt):
        self.init_opt = init_opt
        self.info_dict = {
            'images': None, 'save': None, 'flats': None, 'darks': None, 'biases': None,
            'aavso_num': None, 'second_obs': None, 'date': None, 'lat': None, 'long': None,
            'elev': None, 'camera': None, 'pixel_bin': None, 'filter': None, 'notes': None,
            'plate_opt': None, 'img_align_opt': None, 'tar_coords': None, 'comp_stars': None,
            'prered_file': None, 'file_units': None, 'file_time': None,
            'wl_min': None, 'wl_max': None, 'pixel_scale': None, 'exposure': None
        }
        self.params = {
            'images': imaging_files, 'save': save_directory, 'aavso_num': obs_code, 'second_obs': second_obs_code,
            'date': obs_date, 'lat': latitude, 'long': longitude, 'elev': elevation, 'camera': camera,
            'pixel_bin': pixel_bin, 'filter': filter_type, 'notes': obs_notes, 'plate_opt': plate_solution_opt,
            'img_align_opt': image_align_opt, 'tar_coords': target_star_coords, 'comp_stars': comparison_star_coords
        }

    def complete_red(self, planet):
        for key, value in list(self.params.items()):
            if key == 'elev':
                self.info_dict[key] = self.params[key](self.info_dict[key], self.info_dict['lat'],
                                                       self.info_dict['long'])
            elif key == 'tar_coords':
                self.info_dict[key] = self.params[key](self.info_dict[key], planet)
            elif key == 'comp_stars':
                self.info_dict[key] = self.params[key](self.info_dict[key], False)
            else:
                self.info_dict[key] = self.params[key](self.info_dict[key])
            if key == 'save':
                self.info_dict['flats'], self.info_dict['darks'], self.info_dict['biases'] = \
                    image_calibrations(self.info_dict['flats'], self.info_dict['darks'],
                                       self.info_dict['biases'], self.init_opt)

        return self.info_dict

    def prereduced(self):
        rem_list = ['images', 'plate_opt', 'img_align_opt', 'tar_coords', 'comp_stars']
        [self.params.pop(key) for key in rem_list]

        self.params.update({'exposure': exposure, 'file_units': data_file_units, 'file_time': data_file_time})
        self.info_dict['prered_file'] = prereduced_file(self.info_dict['prered_file'])

        for key, value in list(self.params.items()):
            if key == 'elev':
                self.info_dict[key] = self.params[key](self.info_dict[key], self.info_dict['lat'],
                                                       self.info_dict['long'])
            else:
                self.info_dict[key] = self.params[key](self.info_dict[key])

        return self.info_dict

    def real_time(self, planet):
        rem_list = ['save', 'aavso_num', 'second_obs', 'date', 'lat', 'long', 'elev',
                    'camera', 'pixel_bin', 'filter', 'notes', 'plate_opt', 'img_align_opt']
        [self.params.pop(key) for key in rem_list]

        for key, value in list(self.params.items()):
            if key == 'comp_stars':
                self.info_dict[key] = self.params[key](self.info_dict[key], True)
            elif key == 'tar_coords':
                self.info_dict[key] = self.params[key](self.info_dict[key], planet)
            else:
                self.info_dict[key] = self.params[key](self.info_dict[key])

        return self.info_dict

    def search_init(self, init_file, planet_dict):
        cwd = Path.cwd()

        while True:
            try:
                if not init_file:
                    log_info(f"\nYour current working directory is: {cwd}")
                    log_info(f"Potential initialization files I've found in {cwd} are: ")
                    [log_info(f"\t{file}") for file in cwd.glob('*.json') if file.is_file()]

                    init_file = user_input("\nPlease enter the Directory and Filename of "
                                           "your Initialization File: ", type_=str)
                if init_file == 'ok':
                    init_file = '/Users/rzellem/Documents/EXOTIC/inits.json'
                init_file = Path(init_file)
                planet_params = self.comp_params(init_file, planet_dict)
                return init_file, planet_params
            except (FileNotFoundError, IsADirectoryError) as e:
                log_info(f"*** Error: Initialization file not found. \n{e}. \nPlease try again. ***")

                log_info(f"\nYour current working directory is: {cwd}")
                log_info(f"Potential initialization files I've found in {cwd} are: ")
                [log_info(f"\t{file}") for file in cwd.glob('*.json') if file.is_file()]

                init_file = None

    def comp_params(self, init_file, planet_dict):
        with init_file.open('r') as json_file:
            data = json.load(json_file)

        user_info = {
            'images': 'Directory with FITS files', 'save': 'Directory to Save Plots',
            'flats': 'Directory of Flats', 'darks': 'Directory of Darks', 'biases': 'Directory of Biases',
            'aavso_num': 'AAVSO Observer Code (N/A if none)', 'second_obs': 'Secondary Observer Codes (N/A if none)',
            'date': 'Observation date', 'lat': 'Obs. Latitude', 'long': 'Obs. Longitude',
            'elev': 'Obs. Elevation (meters)', 'camera': 'Camera Type (CCD or DSLR)', 'pixel_bin': 'Pixel Binning',
            'filter': 'Filter Name (aavso.org/filters)', 'notes': 'Observing Notes',
            'plate_opt': 'Plate Solution? (y/n)', 'img_align_opt': 'Align Images? (y/n)',
            'tar_coords': 'Target Star X & Y Pixel', 'comp_stars': 'Comparison Star(s) X & Y Pixel',
        }
        planet_params = {
            'ra': 'Target Star RA', 'dec': 'Target Star Dec', 'pName': "Planet Name", 'sName': "Host Star Name",
            'pPer': 'Orbital Period (days)', 'pPerUnc': 'Orbital Period Uncertainty',
            'midT': 'Published Mid-Transit Time (BJD-UTC)', 'midTUnc': 'Mid-Transit Time Uncertainty',
            'rprs': 'Ratio of Planet to Stellar Radius (Rp/Rs)',
            'rprsUnc': 'Ratio of Planet to Stellar Radius (Rp/Rs) Uncertainty',
            'aRs': 'Ratio of Distance to Stellar Radius (a/Rs)',
            'aRsUnc': 'Ratio of Distance to Stellar Radius (a/Rs) Uncertainty',
            'inc': 'Orbital Inclination (deg)',
            'incUnc': ('Orbital Inclination (deg) Uncertainty', 'Orbital Inclination (deg) Uncertainity'),
            'ecc': 'Orbital Eccentricity (0 if null)', 'teff': 'Star Effective Temperature (K)',
            'teffUncPos': 'Star Effective Temperature (+) Uncertainty',
            'teffUncNeg': 'Star Effective Temperature (-) Uncertainty',
            'met': 'Star Metallicity ([FE/H])', 'metUncPos': 'Star Metallicity (+) Uncertainty',
            'metUncNeg': 'Star Metallicity (-) Uncertainty',
            'logg': 'Star Surface Gravity (log(g))', 'loggUncPos': 'Star Surface Gravity (+) Uncertainty',
            'loggUncNeg': 'Star Surface Gravity (-) Uncertainty'
        }
        opt_info = {
            'prered_file': 'Pre-reduced File:', 'file_time': 'Pre-reduced File Time Format (BJD_TDB, JD_UTC, MJD_UTC)',
            'file_units': 'Pre-reduced File Units of Flux (flux, magnitude, millimagnitude)',
            'wl_min': 'Filter Minimum Wavelength (nm)', 'wl_max': 'Filter Maximum Wavelength (nm)',
            'pixel_scale': 'Pixel Scale (Ex: 5.21 arcsecs/pixel)', 'exposure': 'Exposure Time (s)',
        }

        self.info_dict = init_params(user_info, self.info_dict, data['user_info'])
        self.info_dict = init_params(opt_info, self.info_dict, data['optional_info'])
        return init_params(planet_params, planet_dict, data['planetary_parameters'])


def check_imaging_files(directory, img_type):
    file_extensions = ['.fits', '.fit', '.fts', '.fz']
    input_files = []

    while True:
        try:
            if Path(directory).is_dir() or not directory.replace(' ', '') == '':
                directory = Path(directory)
                for ext in file_extensions:
                    for file in directory.iterdir():
                        if file.is_file() and file.name.lower().endswith(ext.lower()) \
                                and file.name[0:2] not in ('ref', 'wcs'):
                            input_files.append(str(file))
                    if input_files:
                        return input_files
                if not input_files:
                    raise FileNotFoundError
            else:
                raise NotADirectoryError
        except FileNotFoundError:
            log_info(f"\nError: {img_type} files not found with .fits, .fit, .fts, or .fz extensions in {directory}.")
            opt = user_input("\nWould you like to enter in an alternate image extension in addition to .FITS? (y/n): ",
                             type_=str, val1='y', val2='n')
            if opt == 'y':
                add_ext = user_input("Please enter the extension you want to add (EX: .FITS): ", type_=str)
                file_extensions.append(add_ext)
            else:
                directory = user_input(f"Enter the directory path where {img_type} files are located "
                                       f"(Example using the sample data: sample-data/HatP32Dec202017): ", type_=str)
        except (NotADirectoryError, OSError):
            log_info("\nError: No such directory exists when searching for FITS files. Please try again.")
            directory = user_input(f"Enter the directory path where {img_type} files are located "
                                   f"(Example using the sample data: sample-data/HatP32Dec202017): ", type_=str)


def imaging_files(directory):
    if not directory:
        directory = user_input("\nEnter the directory path where imaging files are located. "
                               "(Example using the sample data: sample-data/HatP32Dec202017): ", type_=str)
    return check_imaging_files(directory, 'Imaging')


def save_directory(directory):
    while True:
        try:
            if not directory:
                directory = user_input("\nEnter the directory to Save the Results and Plots into "
                                       "or type new to create one: ", type_=str)
            if directory == 'new':
                directory = create_directory()
            else:
                if not Path(directory).is_dir() or directory.replace(' ', '') == '':
                    raise NotADirectoryError
            return directory
        except (NotADirectoryError, OSError):
            log_info("Error: The directory entered does not exist. Please try again. Make sure to follow this "
                     "\nformatting (using whichever directory you choose): /sample-data/results")
            directory = None


def create_directory():
    save_path = Path.cwd()
    while True:
        directory = user_input("Enter the name for your new directory: ", type_=str)
        try:
            save_path = save_path / directory
            Path(save_path).mkdir()
        except OSError:
            log_info(f"Creation of the directory {save_path}/{directory} failed.")
        else:
            log_info(f"Successfully created the directory {save_path}.")
            return save_path


def image_calibrations(flats_dir, darks_dir, biases_dir, init):
    opt, flats_list, darks_list, biases_list = None, None, None, None

    if init == 'n':
        opt = user_input("\nDo you have any Calibration Images? (Flats, Darks or Biases)? (y/n): ",
                         type_=str, val1='y', val2='n')

    if opt == 'y' or flats_dir:
        flats_list = check_calibration(flats_dir, 'Flats')
    if opt == 'y' or darks_dir:
        darks_list = check_calibration(darks_dir, 'Darks')
    if opt == 'y' or biases_dir:
        biases_list = check_calibration(biases_dir, 'Biases')

    return flats_list, darks_list, biases_list


def check_calibration(directory, image_type):
    if not directory:
        opt = user_input(f"\nDo you have {image_type}? (y/n): ", type_=str, val1='y', val2='n')
        if opt == 'y':
            directory = user_input(f"Please enter the directory path to your {image_type} "
                                   "(must be in their own separate folder): ", type_=str)
    if directory:
        return check_imaging_files(directory, image_type)
    return None


def obs_code(code):
    if code is None:
        code = user_input("Please enter your AAVSO Observer Account Number "
                          "(if none, leave blank and press enter): ", type_=str)
    if not code.replace(' ', ''):
        code = "N/A"
    return code


def second_obs_code(code):
    if code is None:
        code = user_input("Please enter your comma-separated Secondary Observer Codes "
                          "(if none, leave blank and press enter): ", type_=str)
    if not code.replace(' ', ''):
        code = "N/A"
    return code


def obs_date(date):
    while True:
        if not date:
            date = user_input("\nPlease enter the Observation Date: ", type_=str)
        if date:
            break
    if '/' in date:
        date = date.replace('/', '-')
    return date


def latitude(lat):
    while True:
        if not lat:
            lat = user_input("Enter the latitude (in degrees) of where you observed. "
                             "(Don't forget the sign where North is '+' and South is '-')! "
                             "(Example: -32.12): ", type_=str)
        lat = lat.replace(' ', '')

        if lat[0] == '+' or lat[0] == '-':
            # Convert to float if longitude in decimal. If longitude is in +/-HH:MM:SS format, convert to a float.
            try:
                lat = float(lat)
            except ValueError:
                lat = float(dms_to_dd(lat))

            if -90.00 <= lat <= 90.00:
                return lat
            else:
                log_info("Your latitude is out of range. Please enter a latitude between -90 and +90 (deg).")
        else:
            log_info("You forgot the sign for the latitude! North is '+' and South is '-'. Please try again.")
        lat = None


def longitude(long):
    while True:
        if not long:
            long = user_input("Enter the longitude (in degrees) of where you observed. "
                              "(Don't forget the sign where East is '+' and West is '-')! "
                              "(Example: +152.51): ", type_=str)
        long = long.replace(' ', '')

        if long[0] == '+' or long[0] == '-':
            # Convert to float if longitude in decimal. If longitude is in +/-HH:MM:SS format, convert to a float.
            try:
                long = float(long)
            except ValueError:
                long = float(dms_to_dd(long))

            if -180.00 <= long <= 180.00:
                return long
            else:
                log_info("Your longitude is out of range. Please enter a longitude between -180 and +180 (deg).")
        else:
            log_info("You forgot the sign for the longitude! East is '+' and West is '-'. Please try again.")
        long = None


def elevation(elev, lat, long):
    while True:
        try:
            elev = typecast_check(type_=float, val=elev)
            if not elev:
                log_info("\nEXOTIC is retrieving elevation based on entered "
                         "latitude and longitude from Open Elevation.")
                elev = open_elevation(lat, long)
                if not elev:
                    log_info("\nEXOTIC could not retrieve elevation.")
                    elev = user_input("Enter the elevation (in meters) of where you observed: ", type_=float)
            return elev
        except ValueError:
            log_info("The entered elevation is incorrect.")
            elev = None


def camera(c_type):
    if not c_type:
        c_type = user_input("\nPlease enter the camera type (CCD or DSLR): ", type_=str)
    return c_type


def pixel_bin(pix_bin):
    if not pix_bin:
        pix_bin = user_input("Please enter the pixel binning: ", type_=str)
    return pix_bin


def filter_type(f_type):
    if not f_type:
        f_type = user_input("Please enter the filter name: ", type_=str)
    return f_type


def obs_notes(notes):
    if not isinstance(notes, str):
        notes = user_input("Please enter any observing notes (seeing, weather, etc.) or leave blank and press enter: ",
                           type_=str)
    if not notes.replace(' ', ''):
        notes = "na"
    return notes


def plate_solution_opt(opt):
    if opt:
        opt = opt.lower().strip()
    if opt not in ('y', 'n'):
        opt = user_input("\nWould you like to upload the your image for a plate solution?"
                         "\nThis will allow EXOTIC to translate your image's pixels into coordinates on the sky."
                         "\nDISCLAIMER: One of your imaging files will be publicly viewable on "
                         "nova.astrometry.net. (y/n): ", type_=str, val1='y', val2='n')
    return opt


def image_align_opt(opt):
    if opt:
        opt = opt.lower().strip()
    if opt not in ('y', 'n'):
        opt = user_input("\nWould you like to align your images (y/n): ", type_=str, val1='y', val2='n')
    return opt


def target_star_coords(coords, planet):
    if isinstance(coords, list) and len(coords) == 2:
        pass
    else:
        coords = [user_input(f"\nPlease enter {planet}'s X Pixel Coordinate: ", type_=float),
                  user_input(f"\nPlease enter {planet}'s Y Pixel Coordinate: ", type_=float)]

    return coords


def comparison_star_coords(comp_stars, rt_bool):
    if isinstance(comp_stars, list) and 1 <= len(comp_stars) <= 10 and \
            all(isinstance(star, list) for star in comp_stars):
        comp_stars = [star for star in comp_stars if star != []]
    else:
        comp_stars = []

    if not comp_stars:
        while True:
            if not rt_bool:
                num_comp_stars = user_input("\nHow many Comparison Stars would you like to use? (1-10): ", type_=int)
                if 1 <= num_comp_stars <= 10:
                    break
                log_info("\nThe number of Comparison Stars entered is incorrect.")
            else:
                num_comp_stars = 1
                break

        for num in range(num_comp_stars):
            x_pix = user_input(f"\nComparison Star {num + 1} X Pixel Coordinate: ", type_=float)
            y_pix = user_input(f"Comparison Star {num + 1} Y Pixel Coordinate: ", type_=float)
            comp_stars.append([x_pix, y_pix])

    if rt_bool and isinstance(comp_stars[0], list):
        comp_stars = comp_stars[0]

    return comp_stars


def exposure(exp):
    exp = typecast_check(type_=float, val=exp)
    if not exp:
        exp = user_input("Please enter your exposure time (seconds): ", type_=float)
    return exp


def prereduced_file(file):
    while True:
        try:
            if not file:
                file = user_input("Enter the path and file name of your data file: ", type_=str)
            if file == "ok":
                file = "/Users/rzellem/Documents/EXOTIC/sample-data/NormalizedFluxHAT-P-32 bDecember 17, 2017.txt"
                # file = "/Users/rzellem/Downloads/fluxorama.csv
                log_info("Hello, Rob.")

            file = Path(file)

            if file.is_file():
                return file
            else:
                raise FileNotFoundError
        except FileNotFoundError:
            log_info("Error: Data file not found. Please try again.")
            file = None


def data_file_time(time_format):
    log_info("\nNOTE: If your file is not in one of the following formats, "
             "\nplease re-reduce your data into one of the time formats recognized by EXOTIC.")

    while True:
        if not time_format:
            time_format = user_input("\nWhich of the following time formats is your data file stored in? "
                                     "\nBJD_TDB / JD_UTC / MJD_UTC: ", type_=str)
        time_format = time_format.upper().strip()

        if time_format not in ['BJD_TDB', 'JD_UTC', 'MJD_UTC']:
            log_info("Invalid entry; please try again.")
            time_format = None
        else:
            return time_format


def data_file_units(units):
    log_info("\nNOTE: If your file is not in one of the following units, "
             "\nplease re-reduce your data into one of the units of flux recognized by EXOTIC.")

    while True:
        if not units:
            units = user_input("\nWhich of the following units of flux is your data file stored in? "
                               "\nflux / magnitude / millimagnitude: ", type_=str)
        units = units.lower().strip()

        if units not in ['flux', 'magnitude', 'millimagnitude']:
            log_info("Invalid entry; please try again.")
            units = None
        else:
            return units


#temp
def log_info(string):
    print(string)
    log.debug(string)
    return True



# ########################################################################### #
#    Copyright (c) 2019-2020, California Institute of Technology.
#    All rights reserved.  Based on Government Sponsored Research under
#    contracts NNN12AA01C, NAS7-1407 and/or NAS7-03001.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#      1. Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#      2. Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer in
#         the documentation and/or other materials provided with the
#         distribution.
#      3. Neither the name of the California Institute of
#         Technology (Caltech), its operating division the Jet Propulsion
#         Laboratory (JPL), the National Aeronautics and Space
#         Administration (NASA), nor the names of its contributors may be
#         used to endorse or promote products derived from this software
#         without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CALIFORNIA
#    INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#    TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ########################################################################### #
#    EXOplanet Transit Interpretation Code (EXOTIC)
#    # NOTE: See companion file version.py for version info.
# ########################################################################### #

import requests
from json import dumps
from pathlib import Path
from astropy.io.fits import PrimaryHDU, getdata, getheader
from tenacity import retry, retry_if_exception_type, retry_if_result, \
    stop_after_attempt, wait_exponential


def is_false(value):
    return value is False


def result_if_max_retry_count(retry_state):
    pass


class PlateSolution:

    def __init__(self, file=None, directory=None, api_key=None,
                 api_url='http://nova.astrometry.net/api/'):
        if api_key is None:
            api_key = {'apikey': 'vfsyxlmdxfryhprq'}
        self.api_url = api_url
        self.api_key = api_key
        self.file = file
        self.directory = directory

    def plate_solution(self):
        session = self._login()
        if not session:
            return PlateSolution.fail('Login')

        sub_id = self._upload(session)
        if not sub_id:
            return PlateSolution.fail('Upload')

        sub_url = self._get_url(f"submissions/{sub_id}")
        job_id = self._sub_status(sub_url)
        if not job_id:
            return PlateSolution.fail('Submission ID')

        job_url = self._get_url(f"jobs/{job_id}")
        download_url = self.api_url.replace("/api/", f"/wcs_file/{job_id}/")
        wcs_file = Path(self.directory) / "wcs.fits"
        wcs_file = self._job_status(job_url, wcs_file, download_url)
        if not wcs_file:
            return PlateSolution.fail('Job Status')
        else:
            print("WCS file creation successful.")
            return wcs_file

    def _get_url(self, service):
        return self.api_url + service

    @retry(stop=stop_after_attempt(3), wait=wait_exponential(multiplier=1, min=4, max=10),
           retry=(retry_if_result(is_false) | retry_if_exception_type(requests.exceptions.RequestException)),
           retry_error_callback=result_if_max_retry_count)
    def _login(self):
        r = requests.post(self._get_url('login'), data={'request-json': dumps(self.api_key)})
        if r.status_code >= 400:
            return False
        elif r.json()['status'] == 'success':
            return r.json()['session']
        return False

    @retry(stop=stop_after_attempt(3), wait=wait_exponential(multiplier=1, min=4, max=10),
           retry=(retry_if_result(is_false) | retry_if_exception_type(requests.exceptions.RequestException)),
           retry_error_callback=result_if_max_retry_count)
    def _upload(self, session):
        files = {'file': open(self.file, 'rb')}
        headers = {'request-json': dumps({"session": session}), 'allow_commercial_use': 'n',
                   'allow_modifications': 'n', 'publicly_visible': 'n'}

        r = requests.post(self.api_url + 'upload', files=files, data=headers)

        if r.json()['status'] == 'success':
            return r.json()['subid']
        return False

    @retry(stop=stop_after_attempt(20), wait=wait_exponential(multiplier=1, min=4, max=10),
           retry=(retry_if_result(is_false) | retry_if_exception_type(requests.exceptions.RequestException)),
           retry_error_callback=result_if_max_retry_count)
    def _sub_status(self, sub_url):
        r = requests.get(sub_url)
        if r.json()['job_calibrations']:
            return r.json()['jobs'][0]
        return False

    @retry(stop=stop_after_attempt(3), wait=wait_exponential(multiplier=1, min=4, max=10),
           retry=(retry_if_result(is_false) | retry_if_exception_type(requests.exceptions.RequestException)),
           retry_error_callback=result_if_max_retry_count)
    def _job_status(self, job_url, wcs_file, download_url):
        r = requests.get(job_url)
        if r.json()['status'] == 'success':
            r = requests.get(download_url)
            with wcs_file.open('wb') as f:
                f.write(r.content)
            hdu = PrimaryHDU(data=getdata(filename=self.file), header=getheader(filename=wcs_file))
            hdu.writeto(wcs_file, overwrite=True)
            return wcs_file
        return False

    @staticmethod
    def fail(error_type):
        print("WARNING: After multiple attempts, EXOTIC could not retrieve a plate solution from nova.astrometry.net"
              f" due to {error_type}. EXOTIC will continue reducing data without a plate solution.")
        return False



import logging
import requests
from numpy import floor, log10
from tenacity import retry, retry_if_exception_type, retry_if_result, \
    stop_after_attempt, wait_exponential

try:
    from api.plate_solution import is_false, result_if_max_retry_count
except ImportError:
    from .api.plate_solution import is_false, result_if_max_retry_count


log = logging.getLogger(__name__)


def user_input(prompt, type_, val1=None, val2=None, val3=None):
    while True:
        try:
            result = type_(input(prompt))
            log.debug(f"{prompt}{result}")
        except ValueError:
            print("Sorry, not a valid datatype.")
            continue
        if type_ == str and val1 and val2 and val3:
            result = result.lower().strip()
            if result not in (val1, val2, val3):
                print("Sorry, your response was not valid.")
            else:
                return result
        elif type_ == int and val1 and val2 and val3:
            if result not in (val1, val2, val3):
                print("Sorry, your response was not valid.")
            else:
                return result
        elif type_ == int and val1 and val2 and val3:
            if result not in (val1, val2, val3):
                print("Sorry, your response was not valid.")
            else:
                return result
        else:
            return result


def init_params(comp, dict1, dict2):
    for key, value in comp.items():
        try:
            if not isinstance(value, tuple):
                dict1[key] = dict2[value]
            else:
                for val in value:
                    try:
                        dict1[key] = dict2[val]
                    except KeyError:
                        pass
        except KeyError:
            pass
    return dict1


def typecast_check(type_, val):
    try:
        return type_(val)
    except (ValueError, TypeError):
        return False


def round_to_2(*args):
    x = args[0]
    if len(args) == 1:
        y = args[0]
    else:
        y = args[1]
    if floor(y) >= 1. or y == 0.0:
        roundval = 2
    else:
        roundval = -int(floor(log10(abs(y)))) + 1
    return round(x, roundval)


def dms_to_dd(dms_in):
    """
    Quick helper method to convert long/lat values in degree-minute-second (dms) form
    (using ':' separators) to decimal (dd) form
    :param dms_in: DMS long/lat value, colon separated
    :return float: Properly signed long/lat value in decimal float form
    """
    if dms_in is None or isinstance(dms_in, str) is False or str(dms_in).count(":") != 2:
        raise ValueError("Invalid DMS input provided for calculations. ...")
    # clean string of errant leading/trailing/internal spaces
    dms = str(dms_in).strip().replace(" ", "")
    degrees, minutes, seconds = dms.split(":")
    dec = abs(float(degrees)) + float(minutes) / 60. + float(seconds) / 3600.
    if float(degrees) < 0.:
        dec = dec * -1.
    return dec


@retry(stop=stop_after_attempt(3), wait=wait_exponential(multiplier=1, min=4, max=10),
       retry=(retry_if_result(is_false) | retry_if_exception_type(requests.exceptions.RequestException)),
       retry_error_callback=result_if_max_retry_count)
def open_elevation(lat, long):
    query = f"https://api.open-elevation.com/api/v1/lookup?locations={lat},{long}"
    try:
        r = requests.get(query).json()
        return r['results'][0]['elevation']
    except requests.exceptions.RequestException:
        return False



''' Present an interactive function explorer with slider widgets.
Scrub the sliders to change the properties of the ``sin`` curve, or
type into the title text box to update the title of the plot.
Use the ``bokeh serve`` command to run the example by executing:
    bokeh serve interactive_fitter.py
at your command prompt. Then navigate to the URL
http://localhost:5006/interactive_fitter in your browser.'''

import json
import numpy as np
import urllib.request
from bokeh.io import curdoc
from bokeh.layouts import column, row
from bokeh.models import ColumnDataSource, Slider, TextInput
from bokeh.plotting import figure

from exotic.api.elca import transit
from exotic.exotic import NASAExoplanetArchive, get_wcs, find_target

# physical constants
rsun = 6.955e8 #m
msun = 1.989e30 #kg
au = 1.496e11 #m
G = 0.00029591220828559104 # day, AU, Msun

# load data from file
data = json.load(open("sampledata.json", "r"))
data['time'] = np.array(data['time'])

# download priors
target = NASAExoplanetArchive(data["name"])

if target.resolve_name():
    planet_name, _, parameters = target.planet_info()
else:
    raise(Exception(f"can not resolve planet name: {data['name']} in NEA"))

prior = {
    'rprs':parameters['rprs'],     # Rp/Rs
    'ars':parameters['aRs'],       # a/Rs
    'per':parameters['pPer'],      # Period [day]
    'inc':parameters['inc'],       # Inclination [deg]
    'u0': 0, 'u1': 0, 'u2': 0, 'u3': 0,  # limb darkening (nonlinear)
    'ecc': parameters['ecc'],            # Eccentricity
    'omega': 0,                     # Arg of periastron
    'tmid':parameters['midT'],      # time of mid transit [day]
}

print(json.dumps(parameters,indent=4))


# generate transit model
tdata = transit(data['time'], prior)
source_transit = ColumnDataSource(data=dict(x=data['time'], y=tdata))

# format data for bokeh
source_noisy = ColumnDataSource(data=dict(x=data['time'], y=data['flux']))

# Light curve plot
plot = figure(plot_height=400, plot_width=400, title="Transit Light Curve",
              tools="crosshair,pan,reset,save,wheel_zoom",
              y_range=[0.95, 1.01])

plot.line('x', 'y', source=source_transit, line_width=3, line_alpha=0.6)
plot.circle('x', 'y', source=source_noisy, color='black')
plot.xaxis.axis_label = "Time [day]"
plot.yaxis.axis_label = "Relative Flux"


# Sun + Planet plot
plot2 = figure(plot_height=400, plot_width=400, x_range=[-2,2], y_range=[-2, 2], title="Exoplanet Transit Simulator")

#sun
sundata = ColumnDataSource(data={'x':[0], 'y':[0]})
plot2.circle('x', 'y', source=sundata, fill_color='orange', radius=1, line_color='yellow')

#planet
planetdata = ColumnDataSource(data={'x':[0], 'y':[0], 's':[prior['rprs']]})
plot2.circle('x', 'y', source=planetdata, fill_color='black', radius='s', line_color='green')

# Set up widgets
rprs2 = Slider(title="Transit Depth (Rp/Rs)^2", value=prior['rprs']**2, start=0.00, end=0.04, step=0.001)
inc = Slider(title="Inclination (i)", value=prior['inc'], start=80, end=90, step=0.1)
tmid = Slider(title="Mid-Transit (Tmid)", value=prior['tmid'], start=min(data['time']), end=max(data['time']), step=0.001)

# Example API call from DIY Planet Search: https://waps.cfa.harvard.edu/microobservatory/diy/api/v1/datasets/dataset.php?target=tres-3-b
# TODO add text input for a url             (https://docs.bokeh.org/en/latest/docs/user_guide/interaction/widgets.html#textinput)
# TODO add button called "Load from URL"    (https://docs.bokeh.org/en/latest/docs/user_guide/interaction/widgets.html#button)
# TODO create a function that loads a json from the text box url and updates the data plot
#  when updating the plot data, x-axis use key: "timeJD", y-axis use key: "brightness"
# webtext = urllib.request.urlopen(url).read()
# newdata = json.loads(webtext)

def update_data(attrname, old, new):
    prior['rprs'] = rprs2.value**0.5
    prior['inc'] = inc.value
    prior['tmid'] = tmid.value
    offset = prior['ars']*np.cos(np.deg2rad(prior['inc'])) #hypotenuse #b

    # update plot data
    source_transit.data = dict(x=data['time'], y=transit(data['time'], prior))
    planetdata.data = dict(x=[0], y=[offset], s=[prior['rprs']])

for w in [rprs2, inc, tmid]:
    w.on_change('value', update_data)

# TODO set button.on_change to function above

# Set up layouts and add to document
inputs = column(rprs2, tmid, inc) # TODO add text box and button to column
plots = row(plot, plot2)
curdoc().add_root(column(plots, inputs, width=800))
curdoc().title = "Exoplanet Transit"

